package com.yijia.system.service.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import com.yijia.common.core.domain.model.LoginUser;
import com.yijia.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yijia.common.annotation.DataScope;
import com.yijia.common.constant.UserConstants;
import com.yijia.common.core.domain.TreeSelect;
import com.yijia.common.core.domain.entity.SysDept;
import com.yijia.common.core.domain.entity.SysRole;
import com.yijia.common.exception.CustomException;
import com.yijia.common.utils.StringUtils;
import com.yijia.system.mapper.SysDeptMapper;
import com.yijia.system.mapper.SysRoleMapper;
import com.yijia.system.service.ISysDeptService;

/**
 * 部门管理 服务实现
 *
 * @author yijia
 */
@Service
public class SysDeptServiceImpl implements ISysDeptService
{
    @Autowired(required = false)
    private SysDeptMapper deptMapper;

    @Autowired(required = false)
    private SysRoleMapper roleMapper;

    /**
     * 查询部门管理数据
     *
     * @param dept 部门信息
     * @return 部门信息集合
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysDept> selectDeptList(SysDept dept)
    {
        return deptMapper.selectDeptList(dept);
    }

    /**
     * 构建前端所需要树结构
     *
     * @param depts 部门列表
     * @return 树结构列表
     */
    @Override
    public List<SysDept> buildDeptTree(List<SysDept> depts)
    {
        List<SysDept> returnList = new ArrayList<SysDept>();
        List<Long> tempList = new ArrayList<Long>();
        for (SysDept dept : depts)
        {
            tempList.add(dept.getDeptId());
        }
        for (Iterator<SysDept> iterator = depts.iterator(); iterator.hasNext();)
        {
            SysDept dept = (SysDept) iterator.next();
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(dept.getParentId()))
            {
                recursionFn(depts, dept);
                returnList.add(dept);
            }
        }
        if (returnList.isEmpty())
        {
            returnList = depts;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param depts 部门列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildDeptTreeSelect(List<SysDept> depts)
    {
        List<SysDept> deptTrees = buildDeptTree(depts);
        return deptTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 根据角色ID查询部门树信息
     *
     * @param roleId 角色ID
     * @return 选中部门列表
     */
    @Override
    public List<Integer> selectDeptListByRoleId(Long roleId)
    {
        SysRole role = roleMapper.selectRoleById(roleId);
        return deptMapper.selectDeptListByRoleId(roleId, role.isDeptCheckStrictly());
    }

    /**
     * 根据部门ID查询信息
     *
     * @param deptId 部门ID
     * @return 部门信息
     */
    @Override
    public SysDept selectDeptById(Long deptId)
    {
        return deptMapper.selectDeptById(deptId);
    }

    /**
     * 根据ID查询所有子部门（正常状态）
     *
     * @param deptId 部门ID
     * @return 子部门数
     */
    @Override
    public int selectNormalChildrenDeptById(Long deptId)
    {
        return deptMapper.selectNormalChildrenDeptById(deptId);
    }

    /**
     * 是否存在子节点
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public boolean hasChildByDeptId(Long deptId)
    {
        int result = deptMapper.hasChildByDeptId(deptId);
        return result > 0 ? true : false;
    }

    /**
     * 查询部门是否存在用户
     *
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistUser(Long deptId)
    {
        int result = deptMapper.checkDeptExistUser(deptId);
        return result > 0 ? true : false;
    }

    /**
     * 校验部门名称是否唯一
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public String checkDeptNameUnique(SysDept dept)
    {
        Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
        SysDept info = deptMapper.checkDeptNameUnique(dept.getDeptName(), dept.getParentId());
        if (StringUtils.isNotNull(info) && info.getDeptId().longValue() != deptId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 新增保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public int insertDept(SysDept dept)
    {
        SysDept info = deptMapper.selectDeptById(dept.getParentId());
        // 如果父节点不为正常状态,则不允许新增子节点
        if (!UserConstants.DEPT_NORMAL.equals(info.getStatus())){
            throw new CustomException("部门停用，不允许新增");
        }
        if(dept.getJiBie()!=null && dept.getJiBie() == 2){
            dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
            dept.setCardEnabledFlag("0");
            dept.setWsPrintFlag("2");
            dept.setPrintSetting("1");
            dept.setDiscountSetting("0");
            dept.setIntegralPrintFlag("1");
            dept.setIntegralFlag("1");
            dept.setLabelFlag("0");
            dept.setPrintDeviceType("2");
        }else if(dept.getJiBie()!=null && dept.getJiBie()==1){
            dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
            dept.setCardRuleFlag("0");
            dept.setIntegralRuleFlag("0");
            dept.setCouponFlag("0");
            dept.setIntegralFlag("0");
        }
        return deptMapper.insertDept(dept);
    }

    /**
     * 修改保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public int updateDept(SysDept dept)
    {
        SysDept newParentDept = deptMapper.selectDeptById(dept.getParentId());
        SysDept oldDept = deptMapper.selectDeptById(dept.getDeptId());
        if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept))
        {
            String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            String oldAncestors = oldDept.getAncestors();
            dept.setAncestors(newAncestors);
            updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
        }
        int result = deptMapper.updateDept(dept);
        if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()))
        {
            // 如果该部门是启用状态，则启用该部门的所有上级部门
            updateParentDeptStatus(dept);
        }
        return result;
    }

    /**
     * 修改该部门的父级部门状态
     *
     * @param dept 当前部门
     */
    private void updateParentDeptStatus(SysDept dept)
    {
        String updateBy = dept.getUpdateBy();
        dept = deptMapper.selectDeptById(dept.getDeptId());
        dept.setUpdateBy(updateBy);
        deptMapper.updateDeptStatus(dept);
    }

    /**
     * 修改子元素关系
     *
     * @param deptId 被修改的部门ID
     * @param newAncestors 新的父ID集合
     * @param oldAncestors 旧的父ID集合
     */
    public void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors)
    {
        List<SysDept> children = deptMapper.selectChildrenDeptById(deptId);
        for (SysDept child : children)
        {
            child.setAncestors(child.getAncestors().replace(oldAncestors, newAncestors));
        }
        if (children.size() > 0)
        {
            deptMapper.updateDeptChildren(children);
        }
    }

    /**
     * 删除部门管理信息
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public int deleteDeptById(Long deptId)
    {
        return deptMapper.deleteDeptById(deptId);
    }

    @Override
    public List<SysDept> selectDeptListTree(SysDept dept) {
        return deptMapper.selectDeptListTree(dept);
    }

    /**
     * 根据当前deptID 获取对应所有的油站id
     * @param dept
     * @return
     */
    @Override
    public List<String>  selectDeptId(SysDept dept) {
        List<String> stringlist = new ArrayList<>();
        //根据当前的deptId获取当前组织机构的级别
        SysDept sysDept = deptMapper.selectDeptById(dept.getDeptId());
        //根据级别获取对应数据
        if(sysDept !=null &&  sysDept.getJiBie()!=null ){
            if(sysDept.getJiBie()==0){
                SysDept sysDept1 =new SysDept();
                List<SysDept> list1 = deptMapper.selectDeptList(sysDept1);
                for(SysDept dpt1:list1){
                    stringlist.add(dpt1.getDeptId().toString());
                }
            }else {
                SysDept sysDept2 =new SysDept();
                sysDept2.setOrDeptId(sysDept.getDeptId());
                List<SysDept> list2 = deptMapper.selectDeptList(sysDept2);
                for(SysDept dpt2:list2){
                    stringlist.add(dpt2.getDeptId().toString());
                }
            }
        }
        return stringlist;
    }

    /**
     * 根据or 查询
     * @param dept
     * @return
     */
    @Override
    public List<SysDept> queryDeptList(SysDept dept) {
        return deptMapper.queryDeptList(dept);
    }

    @Override
    public SysDept getGroupInfo(SysDept dept) {
        return deptMapper.getGroupInfo(dept);
    }

    /**
     * 是否共享积分规则
     */
    @Override
    public List<String> deptIntegralInfoList(SysDept dept) {
        List<String> list = new ArrayList<>();
        if(dept!=null &&dept.getJiBie()!=null  && dept.getJiBie()==1 ){
            if(dept.getIntegralRuleFlag()!=null &&dept.getIntegralRuleFlag().equals("1")){
                list.add(dept.getDeptId()+"");
            }else if(dept.getIntegralRuleFlag()!=null &&dept.getIntegralRuleFlag().equals("0")){
                SysDept sysDept2 =new SysDept();
                sysDept2.setParentId(dept.getDeptId());
                List<SysDept> list2 = deptMapper.selectDeptList(sysDept2);
                for(SysDept dpt2:list2){
                    list.add(dpt2.getDeptId().toString());
                }
            }
        }else if(dept!=null && dept.getJiBie()!=null  && dept.getJiBie()==2){
            SysDept sysDept  = deptMapper.selectDeptById(dept.getParentId());
            if(sysDept.getIntegralRuleFlag()!=null &&sysDept.getIntegralRuleFlag().equals("1")){
                list.add(sysDept.getDeptId().toString());
            }else if(sysDept.getIntegralRuleFlag()!=null &&sysDept.getIntegralRuleFlag().equals("0")){
                list.add(dept.getDeptId()+"");
            }
        }else if(dept!=null && dept.getJiBie()!=null  && dept.getJiBie()==0) {
            SysDept sysDept =new SysDept();
            List<SysDept> list2 = deptMapper.selectDeptList(sysDept);
            for(SysDept dpt2:list2){
                list.add(dpt2.getDeptId().toString());
            }
        }
        return list;
    }
    /**
     * 是否共享电子卡规则
     */
    @Override
    public List<String> deptCardInfoList(SysDept dept) {
        List<String> list = new ArrayList<>();
        if(dept!=null &&dept.getJiBie()!=null  && dept.getJiBie()==1 ){
            if(dept.getCardRuleFlag()!=null &&dept.getCardRuleFlag().equals("1")){
                list.add(dept.getDeptId()+"");
            }else if(dept.getCardRuleFlag()!=null &&dept.getCardRuleFlag().equals("0")){
                SysDept sysDept2 =new SysDept();
                sysDept2.setParentId(dept.getDeptId());
                List<SysDept> list2 = deptMapper.selectDeptList(sysDept2);
                for(SysDept dpt2:list2){
                    list.add(dpt2.getDeptId().toString());
                }
            }
        }else if(dept!=null && dept.getJiBie()!=null  && dept.getJiBie()==2){
            SysDept sysDept  = deptMapper.selectDeptById(dept.getParentId());
            if(sysDept.getCardRuleFlag()!=null &&sysDept.getCardRuleFlag().equals("1")){
                list.add(sysDept.getDeptId().toString());
            }else if(sysDept.getCardRuleFlag()!=null &&sysDept.getCardRuleFlag().equals("0")){
                list.add(dept.getDeptId()+"");
            }
        }else if(dept!=null && dept.getJiBie()!=null  && dept.getJiBie()==0) {
            SysDept sysDept =new SysDept();
            List<SysDept> list2 = deptMapper.selectDeptList(sysDept);
            for(SysDept dpt2:list2){
                list.add(dpt2.getDeptId().toString());
            }
        }
        return list;
    }

    /**
     * 是否共享优惠劵
     * @param dept
     * @return
     */
    @Override
    public List<String> deptCouponInfoList(SysDept dept) {
        List<String> list = new ArrayList<>();
        if(dept!=null &&dept.getJiBie()!=null  && dept.getJiBie()==1 ){
            if(dept.getCouponFlag()!=null &&dept.getCouponFlag().equals("1")){
                list.add(dept.getDeptId()+"");
            }else if(dept.getCouponFlag()!=null &&dept.getCouponFlag().equals("0")){
                SysDept sysDept2 =new SysDept();
                sysDept2.setParentId(dept.getDeptId());
                List<SysDept> list2 = deptMapper.selectDeptList(sysDept2);
                for(SysDept dpt2:list2){
                    list.add(dpt2.getDeptId().toString());
                }
            }
        }else if(dept!=null && dept.getJiBie()!=null  && dept.getJiBie()==2){
            SysDept sysDept  = deptMapper.selectDeptById(dept.getParentId());
            if(sysDept.getCouponFlag()!=null &&sysDept.getCouponFlag().equals("1")){
                list.add(sysDept.getDeptId().toString());
            }else if(sysDept.getCouponFlag()!=null &&sysDept.getCouponFlag().equals("0")){
                list.add(dept.getDeptId()+"");
            }
        }else if(dept!=null && dept.getJiBie()!=null  && dept.getJiBie()==0) {
            SysDept sysDept =new SysDept();
            List<SysDept> list2 = deptMapper.selectDeptList(sysDept);
            for(SysDept dpt2:list2){
                list.add(dpt2.getDeptId().toString());
            }
        }
        return list;
    }

    /**
     * 修改油站基础信息
     * @param dept
     * @return
     */
    @Override
    public int updateDeptInfo(SysDept dept) {
        return deptMapper.updateDept(dept);
    }

    //查询所有的油站信息，不涉及到优惠卷，电子卡，积分的共享。
    @Override
    public List<String> selectDeptIdList(SysDept dept) {
        List<String> list = new ArrayList<>();
        //根据当前的deptId获取当前组织机构的级别
        SysDept sysDept = deptMapper.selectDeptById(dept.getDeptId());
        //超级管理员级别查询所有的油站和集团
        if(sysDept!=null && sysDept.getJiBie()!=null  && sysDept.getJiBie()==0) {
            SysDept sysDept1 = new SysDept();
            sysDept1.setDelFlag("0");
            List<SysDept> list2 = deptMapper.selectDeptList(sysDept1);
            for(SysDept dpt2:list2){
                list.add(dpt2.getDeptId().toString());
            }
            // 集团级别，查询集团和集团下的所有的站点信息
        }else if(sysDept!=null &&sysDept.getJiBie()!=null  && sysDept.getJiBie()==1 ){
            SysDept sysDept2 =new SysDept();
            sysDept2.setParentId(dept.getDeptId());
            List<SysDept> list2 = deptMapper.selectDeptList(sysDept2);
            for(SysDept dpt2:list2){
                list.add(dpt2.getDeptId().toString());
            }
            // 查询油站信息。
        }else if(sysDept!=null && sysDept.getJiBie()!=null  && sysDept.getJiBie()==2){
            SysDept depts  = deptMapper.selectDeptById(sysDept.getParentId());
            list.add(depts.getDeptId().toString());
        }
        return list;
    }

    /**
     *  查询分配代理管理
     * @param dept
     * @return
     */
    @Override
    public List<SysDept> selectSysAgentList(SysDept dept) {

        return deptMapper.selectSysAgentList(dept);
    }

    @Override
    public List<SysDept> selectDeptAgentList(SysDept sysDept) {
        return deptMapper.selectDeptAgentList(sysDept);
    }

    @Override
    public List<SysDept> searchDeptList(SysDept sysDept) {
        return deptMapper.searchDeptList(sysDept);
    }

    /**
     * 查询当前人的站点信息
     * @return
     */
    @Override
    public List<String> levelDeptList(Long levelId) {
        LoginUser currentUser = SecurityUtils.getLoginUser();
        List<SysDept> deptList = new ArrayList<>();
        List<String> stringList = new ArrayList<>();
        if(levelId!=null){
            //查询油站信息
           SysDept dept = deptMapper.selectDeptById(levelId);
           if(dept!=null && dept.getJiBie()!=null &&dept.getJiBie()==0){
               deptList =deptMapper.selectChildrenDeptById(levelId);
           }else if(dept!=null && dept.getJiBie()!=null && dept.getJiBie()==1){
               deptList =deptMapper.selectGroupAndDeptList(levelId);
           }else if(dept!=null && dept.getJiBie()!=null && dept.getJiBie()==2){
               deptList.add(dept);
           }
        }else{
            if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 0 && currentUser.getUser().getAgentFlag()!=null&& currentUser.getUser().getAgentFlag().equals("0")){
                //顶级 查询处所有的 集团和油站id
                deptList =deptMapper.selectChildrenDeptById(currentUser.getUser().getDeptId());
            }else if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 0 && currentUser.getUser().getAgentFlag()!=null&& currentUser.getUser().getAgentFlag().equals("1")){
                SysDept dept =new SysDept();
                dept.setUserId(currentUser.getUser().getUserId());
                deptList = deptMapper.selectDeptAgentList(dept);
            }else if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 1){
                //查询集团及站点
                deptList =deptMapper.selectGroupAndDeptList(currentUser.getUser().getDeptId());
            }else{
                SysDept dept =new SysDept();
                dept.setDeptId(currentUser.getUser().getDeptId());
                dept.setParentId(currentUser.getUser().getDept().getParentId());
                deptList.add(dept);
            }
        }
        if(deptList!=null && deptList.size()>0){
            for(SysDept sysDept:deptList){
                stringList.add(sysDept.getDeptId()+"");
            }
        }
        return stringList;
    }
    //查询当前人涉及到的站点信息
    public List<String> selectdeptInfoList(SysDept deptinfo){
        LoginUser currentUser = SecurityUtils.getLoginUser();
        List<SysDept> deptList = new ArrayList<>();
        List<String> stringList = new ArrayList<>();
        if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 0 ){
            //顶级 查询处所有的 集团和油站id
            SysDept dept =new SysDept();
            dept.setJiBie(2);
            deptList =deptMapper.selectDeptList(dept);
        }else if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 1){
            //查询集团及站点
            SysDept dept =new SysDept();
            dept.setParentId(currentUser.getUser().getDeptId());
            deptList =deptMapper.selectDeptList(dept);
        }else{
            SysDept dept =new SysDept();
            dept.setDeptId(currentUser.getUser().getDeptId());
            dept.setParentId(currentUser.getUser().getDept().getParentId());
            deptList.add(dept);
        }
        if(deptList!=null && deptList.size()>0){
            for(SysDept sysDept:deptList){
                stringList.add(sysDept.getDeptId()+"");
            }
        }
        return stringList;
    }

    @Override
    public boolean isLngInfoList() {
        LoginUser currentUser = SecurityUtils.getLoginUser();
        boolean flag =false ;
        if(currentUser.getUser().getDept().getJiBie()==1){
            if(currentUser.getUser().getDept().getIsLngFlag()!=null &&currentUser.getUser().getDept().getIsLngFlag().equals("1") ){
                flag = true;
            }
        }else if(currentUser.getUser().getDept().getJiBie()==2){
            SysDept dept =selectDeptById(currentUser.getUser().getDeptId());
            if(dept!=null &&dept.getIsLngFlag()!=null && dept.getIsLngFlag().equals("1")){
                flag = true;
            }
        }
        return flag;
    }

    /**
     * 获取所有的LNG站点数据
     * @param levelId
     * @return
     */
    @Override
    public List<String> levelLngDeptList(Long levelId) {
        LoginUser currentUser = SecurityUtils.getLoginUser();
        List<SysDept> deptList = new ArrayList<>();
        List<String> stringList = new ArrayList<>();
        if(levelId!=null){
            //查询油站信息
            SysDept dept = deptMapper.selectDeptById(levelId);
            if(dept!=null && dept.getJiBie()!=null &&dept.getJiBie()==0){
                SysDept deptGroup =new SysDept();
                deptGroup.setIsLngFlag("1");
                List<SysDept>   deptInfoList =deptMapper.selectDeptList(deptGroup);
                if(deptInfoList!=null && deptInfoList.size()>0){
                    SysDept deptinfo =new SysDept();
                    List<String> deptIdList = new ArrayList<>();
                    for(SysDept sysDept:deptInfoList){
                        deptIdList.add(sysDept.getDeptId().toString());
                    }
                    deptList = deptMapper.selectDeptList(deptinfo);
                }
            }else if(dept!=null && dept.getJiBie()!=null && dept.getJiBie()==1 && dept.getIsLngFlag()!=null && dept.getIsLngFlag().equals("1")){
                deptList =deptMapper.selectChildrenDeptById(dept.getDeptId());
            }else if(dept!=null && dept.getJiBie()!=null && dept.getJiBie()==2){
                SysDept dept1 =deptMapper.selectDeptById(dept.getParentId());
                if(dept1.getIsLngFlag()!=null && dept1.getIsLngFlag().equals("1")){
                    deptList.add(dept);
                }
            }
        }else{
            if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 0 && currentUser.getUser().getAgentFlag()!=null&& currentUser.getUser().getAgentFlag().equals("0")){
                //顶级 查询处所有的 集团和油站id
                SysDept deptGroup =new SysDept();
                deptGroup.setIsLngFlag("1");
                List<SysDept>   deptInfoList =deptMapper.selectDeptList(deptGroup);
                if(deptInfoList!=null && deptInfoList.size()>0){
                    SysDept deptinfo =new SysDept();
                    List<String> deptIdList = new ArrayList<>();
                    for(SysDept sysDept:deptInfoList){
                        deptIdList.add(sysDept.getDeptId().toString());
                    }
                    deptList = deptMapper.selectDeptList(deptinfo);
                }
            }else if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 0 && currentUser.getUser().getAgentFlag()!=null&& currentUser.getUser().getAgentFlag().equals("1")){
                SysDept dept =new SysDept();
                dept.setIsLngFlag("1");
                dept.setUserId(currentUser.getUser().getUserId());
                List<SysDept>   deptInfoList = deptMapper.selectDeptAgentList(dept);
                if(deptInfoList!=null && deptInfoList.size()>0){
                    SysDept deptinfo =new SysDept();
                    List<String> deptIdList = new ArrayList<>();
                    for(SysDept sysDept:deptInfoList){
                        deptIdList.add(sysDept.getDeptId().toString());
                    }
                    deptList = deptMapper.selectDeptList(deptinfo);
                }
            }else if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 1&& currentUser.getUser().getDept().getIsLngFlag()!=null && currentUser.getUser().getDept().getIsLngFlag().equals("1")){
                //查询集团及站点
                deptList =deptMapper.selectChildrenDeptById(currentUser.getUser().getDeptId());
            }else{
                SysDept dept1 =deptMapper.selectDeptById(currentUser.getUser().getDept().getParentId());
                if(dept1.getIsLngFlag()!=null && dept1.getIsLngFlag().equals("1")){
                    SysDept dept =new SysDept();
                    dept.setDeptId(currentUser.getUser().getDeptId());
                    dept.setParentId(currentUser.getUser().getDept().getParentId());
                    deptList.add(dept);
                }
            }
        }
        if(deptList!=null && deptList.size()>0){
            for(SysDept sysDept:deptList){
                stringList.add(sysDept.getDeptId()+"");
            }
        }
        return stringList;
    }

    /**
     * 查询代理所有的集团
     * @param sysDept
     * @return
     */
    @Override
    public List<SysDept> selectGroupAgentList(SysDept sysDept) {
        return deptMapper.selectGroupAgentList(sysDept);
    }

    /**
     * 根据当前人查询集团是否共享电子卡
     * @param levelId
     * @return
     */
    @Override
    public List<String> levelShareCardList(Long levelId) {
        LoginUser currentUser = SecurityUtils.getLoginUser();
        List<SysDept> deptList = new ArrayList<>();
        List<String> stringList = new ArrayList<>();
        if(levelId!=null){
            //查询油站信息
            SysDept dept = deptMapper.selectDeptById(levelId);
            if(dept!=null && dept.getJiBie()!=null &&dept.getJiBie()==0){
                //查询出所有共享电子卡的集团
                SysDept sysDept = new SysDept();
                sysDept.setJiBie(1);
                sysDept.setCardRuleFlag("1");
                List<SysDept>  shareGroupList =deptMapper.selectShareGroup(sysDept);
                if(shareGroupList!=null && shareGroupList.size()>0){
                    deptList.addAll(shareGroupList);
                }
                //查询所有不共享的站点
                List<SysDept>  notShareDeptList =deptMapper.selectNotShareDept(sysDept);
                if(notShareDeptList!=null && notShareDeptList.size()>0){
                    deptList.addAll(notShareDeptList);
                }
            }else if(dept!=null && dept.getJiBie()!=null && dept.getJiBie()==1){
                //判断集团电子卡共享
                if(dept.getCardRuleFlag()!=null && dept.getCardRuleFlag().equals("1")){
                    deptList.add(dept);
                }else{
                    //判断集团电子卡不共享，查询出所有的站点
                    SysDept sysDept = new SysDept();
                    sysDept.setParentId(dept.getDeptId());
                    List<SysDept>   sysList =deptMapper.searchDeptList(sysDept);
                    if(sysList!=null &&sysList.size()>0){
                        deptList.addAll(sysList);
                    }
                }
                deptList =deptMapper.selectGroupAndDeptList(levelId);
            }else if(dept!=null && dept.getJiBie()!=null && dept.getJiBie()==2){
                deptList.add(dept);
            }
        }else{
            if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 0 && currentUser.getUser().getAgentFlag()!=null&& currentUser.getUser().getAgentFlag().equals("0")){
                //顶级 查询处所有的 集团和油站id
                //查询出所有共享电子卡的集团
                SysDept sysDept = new SysDept();
                sysDept.setJiBie(1);
                sysDept.setCardRuleFlag("1");
                List<SysDept>  shareGroupList =deptMapper.selectShareGroup(sysDept);
                if(shareGroupList!=null && shareGroupList.size()>0){
                    deptList.addAll(shareGroupList);
                }
                //查询所有不共享的站点
                List<SysDept>  notShareDeptList =deptMapper.selectNotShareDept(sysDept);
                if(notShareDeptList!=null && notShareDeptList.size()>0){
                    deptList.addAll(notShareDeptList);
                }
            }else if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 0 && currentUser.getUser().getAgentFlag()!=null&& currentUser.getUser().getAgentFlag().equals("1")){
                SysDept dept =new SysDept();
                dept.setUserId(currentUser.getUser().getUserId());
                //查询出代理人所有的集团信息
               List<SysDept> deptAgentList = deptMapper.selectGroupAgentList(dept);
                //查询出共享的集团
                if(deptAgentList!=null && deptAgentList.size()>0){
                    for(SysDept deptAgent:deptAgentList){
                        if(deptAgent.getCardRuleFlag()!=null && deptAgent.getCardRuleFlag().equals("1")){
                            deptList.add(dept);
                        }else if(deptAgent.getCardRuleFlag()!=null && deptAgent.getCardRuleFlag().equals("0")){
                            //查询不共享的集团下的所有油站
                            SysDept sysDept = new SysDept();
                            sysDept.setParentId(deptAgent.getDeptId());
                            List<SysDept>   sysList =deptMapper.searchDeptList(sysDept);
                            if(sysList!=null &&sysList.size()>0){
                                deptList.addAll(sysList);
                            }
                        }
                    }
                }
            }else if(currentUser.getUser().getDept().getJiBie()!=null && currentUser.getUser().getDept().getJiBie() == 1){
                //判断集团电子卡共享
                if( currentUser.getUser().getDept().getCardRuleFlag()!=null &&  currentUser.getUser().getDept().getCardRuleFlag().equals("1")){
                    deptList.add(currentUser.getUser().getDept());
                }else if(currentUser.getUser().getDept().getCardRuleFlag()!=null &&  currentUser.getUser().getDept().getCardRuleFlag().equals("0")){
                    //判断集团电子卡不共享，查询出所有的站点
                    SysDept sysDept = new SysDept();
                    sysDept.setParentId(currentUser.getUser().getDeptId());
                    List<SysDept>   sysList =deptMapper.searchDeptList(sysDept);
                    if(sysList!=null &&sysList.size()>0){
                        deptList.addAll(sysList);
                    }
                }
            }else{
                SysDept dept =new SysDept();
                dept.setDeptId(currentUser.getUser().getDeptId());
                dept.setParentId(currentUser.getUser().getDept().getParentId());
                deptList.add(dept);
            }
        }
        if(deptList!=null && deptList.size()>0){
            for(SysDept sysDept:deptList){
                stringList.add(sysDept.getDeptId()+"");
            }
        }
        return stringList;
    }

    /**
     * 递归列表
     */
    private void recursionFn(List<SysDept> list, SysDept t)
    {
        // 得到子节点列表
        List<SysDept> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysDept tChild : childList)
        {
            if (hasChild(list, tChild))
            {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysDept> getChildList(List<SysDept> list, SysDept t)
    {
        List<SysDept> tlist = new ArrayList<SysDept>();
        Iterator<SysDept> it = list.iterator();
        while (it.hasNext())
        {
            SysDept n = (SysDept) it.next();
            if (StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getDeptId().longValue())
            {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SysDept> list, SysDept t)
    {
        return getChildList(list, t).size() > 0 ? true : false;
    }
}
